﻿using System;
using System.Windows.Forms;
using InfoAPILib;
using System.IO;
using InfoAPIDemoMATH;
using InfoAPIDemoGETTABLE;
using InfoAPIDemoSENDTABLE;
using InfoAPIDemoGETBAZE;
using InfoAPIDemoATTACHMENTS;

namespace InfoAPIDemo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            cbMathFunction.SelectedIndex = 0;
        }

        private InfoAPIHandler APIHandler;
        private InfoAPIParams APIParams;
        //private InfoAPIResult APIResult;



        private void PrepareAPICall()
        {

            // podešavanje hendlera API konekcije
            this.APIHandler = new InfoAPIHandler();
            this.APIHandler.APIServerURL = tbServerURL.Text;
            this.APIHandler.username = tbUserName.Text;
            this.APIHandler.password = tbPassword.Text;
            this.APIHandler.client_id = "DEMO";
            this.APIHandler.client_pk = "87oluavs978bl94vkivfzs967zs3rf";
            this.APIHandler.client_version = "0.0.1";
            this.APIHandler.poslovna_godina = tbPoslovnaGodina.Text;
            //this.APIHandler.DebugLevel = 0;
            //this.APIHandler.DebugSections = "SYSTEM,CLASS";
            this.APIHandler.SerializeIdented = true;
            //this.APIHandler.SerializeIdented = false;

            this.APIParams.result_database_format = cbDatabaseAttachmentFormat.Text;

        }

        //private InfoAPIResponse ExecuteAPI(InfoAPIParams pAPIParams, InfoAPIResult pAPIResult)
        private InfoAPIResponse ExecuteAPI(InfoAPIParams pAPIParams)
        {

            this.ResetForm();

            this.APIParams = pAPIParams;
            //this.APIResult = pAPIResult;
            this.PrepareAPICall();

            // podizi exception ako dodje do greske
            this.APIHandler.TriggerInfoAPICallException = true;

            this.APIHandler.APICallBeforeCall += this.onBeforeCall;

            // dodaj Class factory za InfoAPILib
            //this.APIHandler.InfoAPIClassFactories.Add(new InfoAPIClassFactory());

            // dodaj Class factory za modul MATH
            this.APIHandler.InfoAPIClassFactories.Add(new MATHClassFactory());

            // dodaj Class factory za modul GETTABLE
            this.APIHandler.InfoAPIClassFactories.Add(new GETTABLEClassFactory());

            // dodaj Class factory za modul SENDTABLE
            this.APIHandler.InfoAPIClassFactories.Add(new SENDTABLEClassFactory());

            // dodaj Class factory za modul GETBAZE
            this.APIHandler.InfoAPIClassFactories.Add(new GETBAZEClassFactory());

            // dodaj Class factory za modul ATTAHCMENTS
            this.APIHandler.InfoAPIClassFactories.Add(new ATTACHMENTSClassFactory());


            tsslStatus.Text = "Sending API request";
            Application.DoEvents();

            // IZVRSI POZIV API FUNKCIJE
            // u mAPIResponse se nalazi odgovor API-ja. U mAPIResponse.Result je reyultat izvrsenja funkcije.

            InfoAPIResponse mAPIResponse = null;

            try
            {
                //mAPIResponse = APIHandler.CallAPI(pAPIParams, pAPIResult);
                mAPIResponse = APIHandler.CallAPI(pAPIParams);

                tbResponse.Text = "Odgovor od API-ja:\r\n" + APIHandler.ResponseString;
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResponse.Result.Serialize();
                tsslStatus.Text = "Received API response";
            }
            catch (InfoAPICallException e)
            {
                tbResult.Text = "Greska:\r\n" + e.ErrorInfo.error_state + "\r\n" + e.ErrorInfo.error_message;
                tsslStatus.Text = "API call error!";
            }
            finally
            {

            }

            return mAPIResponse;
        }


        private void ExecuteAPIAsync(InfoAPIParams pAPIParams, InfoAPICallCompletedEventHandler EventHandler)
        {

            this.ResetForm();

            this.APIParams = pAPIParams;
            
            this.PrepareAPICall();

            // podizi exception ako dodje do greske
            this.APIHandler.TriggerInfoAPICallException = true;

            this.APIHandler.APICallBeforeCall += this.onBeforeCall;

            // Prikaci gnericki hendler koji se izvršava kada se završi poziv API-ju
            this.APIHandler.APICallCompleted += this.onAsyncCompletedEventHandler;

            // Prikaci korisnicki hendler koji se izvršava kada se završi poziv API-ju
            this.APIHandler.APICallCompleted += EventHandler;

            // Prikaci hendler kji se izvršava ako dođe do greške u komunikaciji
            this.APIHandler.APICallError += this.onAsyncErrorEventHandler;

            tsslStatus.Text = "Sending API request (async)";
            Application.DoEvents();


            // IZVRSI ASINHRONI POZIV API FUNKCIJE
            this.APIHandler.CallAPIAsync(pAPIParams);
        }

        private void onBeforeCall(object sender, InfoAPICallBeforeCallEventArgs e)
        {
            tbParams.Text = "Parametri API funkcije:\r\n" + this.APIHandler.ParamsString;
            tbRequest.Text = "Ceo upit koji se salje API-ju:\r\n" + this.APIHandler.QueryString;
            File.WriteAllText("query.json", this.APIHandler.QueryString);
            Application.DoEvents();
        }

        private void ResetForm()
        {
            tbParams.Text = "";
            tbRequest.Text = "";
            tbResponse.Text = "";
            tbResult.Text = "";
            tsslStatus.Text = "";
            Application.DoEvents();
        }

        private void onAsyncCompletedEventHandler(object sender, InfoAPICallCompletedEventArgs pEventArgs)
        {

            tsslStatus.Text = "Received API response";

            InfoAPIResponse mAPIResponse = pEventArgs.InfoAPIResponse;

            if (mAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResponse.Result.Serialize();
            }

            // prikaz prolaznih podataka u tekstboksove radi dibaga
            tbParams.Text = "Parametri API funkcije:\r\n" + APIHandler.ParamsString;
            tbRequest.Text = "Ceo upit koji se salje API-ju:\r\n" + APIHandler.QueryString;
            tbResponse.Text = "Odgovor od API-ja:\r\n" + APIHandler.ResponseString;

        }


        private void onAsyncErrorEventHandler(object sender, InfoAPICallErrorEventArgs pEventArgs)
        {
            tsslStatus.Text = "API call error!";

            tbParams.Text = "Parametri API funkcije:\r\n" + APIHandler.ParamsString;
            tbRequest.Text = "Ceo upit koji se salje API-ju:\r\n" + APIHandler.QueryString;
            tbResponse.Text = "Odgovor od API-ja:\r\n" + APIHandler.ResponseString;
            tbResult.Text = "Greska kod izvrsenja API funkcije!\r\n";
            tbResult.Text += pEventArgs.error_state + "\r\n" + pEventArgs.error_message;
        }

        #region INFO
        private void bntTestINFO_Click(object sender, EventArgs e)
        {

            // PRIPREMA SLANJA UPITA ZA API FUNKCIJU INFO
            // Funkcija nema parametre.

            // kreiraj objekat za parametre funkcije
            InfoAPIParamsINFO mAPIParamsINFO = new InfoAPIParamsINFO();

            // kreiraj objekat za rezultat izvrsenja funkcije
            //InfoAPIResultINFO mAPIResultINFO = new InfoAPIResultINFO();


            // izvrsi upit na API server
            if (cbAsync.Checked)
            {
                //this.ExecuteAPIAsync(mAPIParamsINFO, mAPIResultINFO, onINFOAsyncCompletedEventHandler);
                this.ExecuteAPIAsync(mAPIParamsINFO, onINFOAsyncCompletedEventHandler);
                //throw new Exception("Asinhroni poziv APF_INFO nije implementiran!");
            }
            else
            {
                //InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParamsINFO, mAPIResultINFO);
                InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParamsINFO);
            }

        }


        private void onINFOAsyncCompletedEventHandler(object sender, InfoAPICallCompletedEventArgs pEventArgs)
        {
            InfoAPIResponse mAPIResponse = pEventArgs.InfoAPIResponse;

            InfoAPIResultINFO mAPIResult = (InfoAPIResultINFO)mAPIResponse.Result;

            if (mAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResult.Serialize();
            }
            else
            {
                tbResult.Text = "Greska:\r\n" + mAPIResponse.error_state + "\r\n" + mAPIResponse.error_message;
            }

        }

        #endregion

        #region ATTACHMENTS
        private void onATTACHMENTSAsyncCompletedEventHandler(object sender, InfoAPICallCompletedEventArgs pEventArgs)
        {
            InfoAPIResponse mAPIResponse = pEventArgs.InfoAPIResponse;

            InfoAPIResultATTACHMENTS mAPIResult = (InfoAPIResultATTACHMENTS)mAPIResponse.Result;

            if (mAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResult.Serialize();
            }
            else
            {
                tbResult.Text = "Greska:\r\n" + mAPIResponse.error_state + "\r\n" + mAPIResponse.error_message;
            }

        }


        private void ATTACHMENTSResponseHandler(InfoAPIResponse pAPIResponse)
        {

            InfoAPIResultATTACHMENTS mAPIResult = (InfoAPIResultATTACHMENTS)pAPIResponse.Result;

            if (pAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResult.Serialize();

                InfoAPIAttachments mAttachments = mAPIResult.GetAttachments();
                // snimi attachmente u datoteke.
                foreach (InfoAPIAttachment mAttachment in mAttachments.items)
                {
                    mAttachment.Save ("");
                    tsslStatus.Text += " - Attachment saved: " + mAttachment.id;
                }
                



            }
            else
            {
                tbResult.Text = "Greska:\r\n" + pAPIResponse.error_state + "\r\n" + pAPIResponse.error_message;
            }


        }

        #endregion

        #region MATH
        private void btnTestMATH_Click(object sender, EventArgs e)
        {

            // kreiraj objekat za parametre funkcije MATH
            InfoAPIParamsMATH mAPIParams = new InfoAPIParamsMATH();
            mAPIParams.function = cbMathFunction.SelectedItem.ToString();
            mAPIParams.param1 = float.Parse(tbMATHParam1.Text);
            mAPIParams.param2 = float.Parse(tbMATHParam2.Text);

            // kreiraj objekat za rezultat izvrsenja funkcije
            //InfoAPIResultMATH mAPIResult = new InfoAPIResultMATH();


            // izvrsi upit na API server
            if (cbAsync.Checked)
            {
                //this.ExecuteAPIAsync(mAPIParams, mAPIResult, this.onMATHAsyncCompletedEventHandler);
                this.ExecuteAPIAsync(mAPIParams, this.onMATHAsyncCompletedEventHandler);
            }
            else
            {
                //InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParams, mAPIResult);
                InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParams);
            }

        }

        private void onMATHAsyncCompletedEventHandler(object sender, InfoAPICallCompletedEventArgs pEventArgs)
        {
            /*
            InfoAPIResponse mAPIResponse = pEventArgs.InfoAPIResponse;

            InfoAPIResultMATH mAPIResult = (InfoAPIResultMATH)mAPIResponse.Result;

            if (mAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResult.Serialize();
            }
            else
            {
                tbResult.Text = "Greska:\r\n" + mAPIResponse.error_state + "\r\n" + mAPIResponse.error_message;
            }

            float mValue = ((InfoAPIResultMATH)mAPIResponse.Result).value;
            */

        }
        #endregion

        #region GETBAZE
        private void btnGetBaze_Click(object sender, EventArgs e)
        {
            // PRIPREMA SLANJA UPITA ZA API FUNKCIJU INFO
            // Funkcija nema parametre.

            // kreiraj objekat za parametre funkcije
            InfoAPIParamsGETBAZE mAPIParams = new InfoAPIParamsGETBAZE();
            //mAPIParams.result_database_format = "SQLITE";
            mAPIParams.ArtikliSif = "N";
            mAPIParams.Artikli = "N";
            mAPIParams.Komitenti = "N";
            mAPIParams.Vrste_Dok = "D";

            // kreiraj objekat za rezultat izvrsenja funkcije
            //InfoAPIResultGETBAZE mAPIResult = new InfoAPIResultGETBAZE();

            // izvrsi upit na API server
            if (cbAsync.Checked)
            {
                //this.ExecuteAPIAsync(mAPIParams, mAPIResult, this.onGETBAZEAsyncCompletedEventHandler);
                this.ExecuteAPIAsync(mAPIParams, this.onGETBAZEAsyncCompletedEventHandler);
            }
            else
            {
                //InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParams, mAPIResult);
                InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParams);
                mResponse.Result.SaveDataBaseAttachmentToFile("GETBAZE-data.sqlite");
            }
        }


        private void onGETBAZEAsyncCompletedEventHandler(object sender, InfoAPICallCompletedEventArgs pEventArgs)
        {
            InfoAPIResponse mAPIResponse = pEventArgs.InfoAPIResponse;

            InfoAPIResultGETBAZE mAPIResult = (InfoAPIResultGETBAZE)mAPIResponse.Result;

            if (mAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResult.Serialize();
            }
            else
            {
                tbResult.Text = "Greska:\r\n" + mAPIResponse.error_state + "\r\n" + mAPIResponse.error_message;
            }


            //UPIS U FAJL
            string mDataFileName = "GETBAZE-data." + mAPIResult.database_format + "." + mAPIResult.GetDatabaseFileExtension();
            mAPIResult.SaveDataBaseAttachmentToFile(mDataFileName);
            //mAPIResult.SaveDataBaseAttachmentToFile("GETBAZE-data.sqlite");

        }

        #endregion

        #region GETTABLE
        private void btnGetTable_Click(object sender, EventArgs e)
        {
            InfoAPIParamsGETTABLE mAPIParams = new InfoAPIParamsGETTABLE();
            //mAPIParams.result_database_format = "SQLITE";

            mAPIParams.table_name = "tabela";

            //InfoAPIResultGETTABLE mAPIResult = new InfoAPIResultGETTABLE();

            if (cbAsync.Checked)
            {
                //this.ExecuteAPIAsync(mAPIParams, mAPIResult, this.onGETTABLEAsyncCompletedEventHandler);
                this.ExecuteAPIAsync(mAPIParams, this.onGETTABLEAsyncCompletedEventHandler);
            }
            else
            {
                //InfoAPIResponse mAPIResponse = this.ExecuteAPI(mAPIParams, mAPIResult);
                InfoAPIResponse mAPIResponse = this.ExecuteAPI(mAPIParams);
                if (mAPIResponse != null)
                {
                    this.GETTABLEResponseHandler(mAPIResponse);
                }

            }

        }

        private void onGETTABLEAsyncCompletedEventHandler(object sender, InfoAPICallCompletedEventArgs pEventArgs)
        {
            InfoAPIResponse mAPIResponse = pEventArgs.InfoAPIResponse;
            this.GETTABLEResponseHandler(mAPIResponse);
        }

        private void GETTABLEResponseHandler(InfoAPIResponse pAPIResponse)
        {

            InfoAPIResultGETTABLE mAPIResult = (InfoAPIResultGETTABLE)pAPIResponse.Result;

            if (pAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResult.Serialize();

                // snimi bazu u datoteku. Ekstenzija u imenu datoteke nema znacenje, moze biti i drugi format pa i PLAIN_OBJ.
                string mDataFileName = "GETTABLE-data." + mAPIResult.database_format + "." + mAPIResult.GetDatabaseFileExtension();
                mAPIResult.SaveDataBaseAttachmentToFile(mDataFileName);

                tsslStatus.Text += " - Data saved to: " + mDataFileName;

                // ako je PLAIN_OBJ onda treba da prepisemo podatke u eksternu tabelu
                if (mAPIResult.database_format == "PLAIN_OBJ")
                {

                    //kreiraj novu tabelu u koju treba prepisati podatke iz JSON forata
                    InfoAPIDemoGETTABLE.ExampleTable mTable = new InfoAPIDemoGETTABLE.ExampleTable();

                    mAPIResult.ReadTable("tabela", mTable);

                    Browser mBrowser = new Browser(mTable);
                    mBrowser.ShowDialog();

                }

            }
            else
            {
                tbResult.Text = "Greska:\r\n" + pAPIResponse.error_state + "\r\n" + pAPIResponse.error_message;
            }


        }
        #endregion


        #region SENDTABLE
        private void btnSendTable_Click(object sender, EventArgs e)
        {

            InfoAPIParamsSENDTABLE mAPIParams = new InfoAPIParamsSENDTABLE();

            mAPIParams.database_format = cbDatabaseAttachmentFormat.Text;

            switch (mAPIParams.database_format) {
                case "PLAIN_OBJ":

                    // kreiraj primer tabelarnih podataka (bilo sta sto ima IEnumerable interfejs)
                    // u ovom primaru je ExampleTable naslednik TList
                    InfoAPIDemoSENDTABLE.ExampleTable lExampleTable = new InfoAPIDemoSENDTABLE.ExampleTable();

                    Button mButton = sender as Button;

                    // ako je klikunutod dugme btnSentTable onda napuni tabelu a ako je btnSendEmptyTable ond aposalji praznu
                    if (mButton.Name == "btnSendTable") {
                        // generisi primer podataka u listi
                        lExampleTable.Populate();
                    }

                    //mAPIParams.AddTableReference("tabela", lExampleTable);
                    mAPIParams.AddTableReference<InfoAPIDemoSENDTABLE.ExampleTableDTO>("tabela", lExampleTable);

                    break;

                case "SQLITE":
                    mAPIParams.LoadDatabaseAttachmentFromFile("SENDTABLE-data.sqlite");
                    break;

                case "ZIPDBF":
                    mAPIParams.LoadDatabaseAttachmentFromFile("SENDTABLE-data.ZIPDBF.zip");
                    break;

            }



            //InfoAPIResultSENDTABLE mAPIResult = new InfoAPIResultSENDTABLE();

            if (cbAsync.Checked)
            {
                //this.ExecuteAPIAsync(mAPIParams, mAPIResult, this.onSENDTABLEAsyncCompletedEventHandler);
                this.ExecuteAPIAsync(mAPIParams, this.onSENDTABLEAsyncCompletedEventHandler);
            }
            else
            {
                //InfoAPIResponse mAPIResponse = this.ExecuteAPI(mAPIParams, mAPIResult);
                InfoAPIResponse mAPIResponse = this.ExecuteAPI(mAPIParams);
                if (mAPIResponse != null)
                {
                    this.SENDTABLEResponseHandler(mAPIResponse);
                }

            }

        }

        private void onSENDTABLEAsyncCompletedEventHandler(object sender, InfoAPICallCompletedEventArgs pEventArgs)
        {
            InfoAPIResponse mAPIResponse = pEventArgs.InfoAPIResponse;

            this.SENDTABLEResponseHandler(mAPIResponse);
        }


        private void SENDTABLEResponseHandler(InfoAPIResponse pAPIResponse)
        {

            InfoAPIResultSENDTABLE mAPIResult = (InfoAPIResultSENDTABLE)pAPIResponse.Result;

            if (pAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                tbResult.Text = "Rezultat izvrsenja API funkcije:\r\n" + mAPIResult.Serialize();

            }
            else
            {
                tbResult.Text = "Greska:\r\n" + pAPIResponse.error_state + "\r\n" + pAPIResponse.error_message;
            }

        }
        #endregion

        private void btnTestATTACHMENTS_Click(object sender, EventArgs e)
        {
            // PRIPREMA SLANJA UPITA ZA API FUNKCIJU ATTACHMENTS
            // Funkcija nema parametre.

            // kreiraj objekat za parametre funkcije
            InfoAPIParamsATTACHMENTS mAPIParamsATTACHMENTS = new InfoAPIParamsATTACHMENTS();

            // kreiraj objekat za rezultat izvrsenja funkcije
            //InfoAPIResultINFO mAPIResultINFO = new InfoAPIResultINFO();


            // izvrsi upit na API server
            if (cbAsync.Checked)
            {
                //this.ExecuteAPIAsync(mAPIParamsINFO, mAPIResultINFO, onINFOAsyncCompletedEventHandler);
                this.ExecuteAPIAsync(mAPIParamsATTACHMENTS, onATTACHMENTSAsyncCompletedEventHandler);
                //throw new Exception("Asinhroni poziv APF_INFO nije implementiran!");
            }
            else
            {
                //InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParamsINFO, mAPIResultINFO);
                InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParamsATTACHMENTS);

                if (mResponse != null)
                {
                    this.ATTACHMENTSResponseHandler(mResponse);
                }

            }
        }
    }
}
