﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InfoAPILib;
using System.Diagnostics;

namespace InfoAPIDemoGETBAZE
{
    class Program
    {
        private InfoAPIHandler APIHandler;
        private InfoAPIParams APIParams;
        private RepositoryBaze Repository = null;
        private string TableFormat;

        static void Main(string[] args)
        {
            Stopwatch sw = new Stopwatch();
            Program program = new Program();

            program.PrepareInput(out program.TableFormat);

            sw.Start();
            program.RunTest();
            Console.WriteLine("\nBenchmark: " + sw.Elapsed.TotalSeconds + " SEC");
            Console.ReadLine();
        }

        public void PrepareInput(out string pFormat)
        {
            do
            {
                Console.WriteLine("Odaberite format:\n1. PLAIN_OBJ\n2. SQLLITE\n3. ZIPDBF\n");

                pFormat = Console.ReadLine();
                pFormat = pFormat.ToLower();
                switch (pFormat)
                {
                    case "1":
                    case "plain_obj":
                        pFormat = "PLAIN_OBJ";
                        break;
                    case "2":
                    case "sqllite":
                        pFormat = "SQLITE";
                        break;
                    case "3":
                    case "zipdf":
                        pFormat = "ZIPDBF";
                        break;
                    default:
                        pFormat = "";
                        break;
                }
            } while (pFormat == "");
        }

        public void RunTest()
        {
            InfoAPIParamsGETBAZE mAPIParams = new InfoAPIParamsGETBAZE();
            mAPIParams.ArtikliSif = "N";
            mAPIParams.Artikli = "N";
            mAPIParams.Komitenti = "D";
            mAPIParams.Vrste_Dok = "N";
            InfoAPIResponse mAPIResponse = this.ExecuteAPI(mAPIParams);


            InfoAPIResultGETBAZE mAPIResult = (InfoAPIResultGETBAZE)mAPIResponse.Result;

            if (mAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
            {
                // STAMPA ukoliko su PLAIN_OBJ odstampati kao tabelu
                if(mAPIResult.database_format == "PLAIN_OBJ")
                {
                    Repository = new RepositoryBaze();

                    mAPIResult.ReadTable("komitenti", Repository.Komitenti);
                    mAPIResult.ReadTable("mesta_isporuke", Repository.MestaIsporuke);

                    if (mAPIResult.error_state == "ok")
                    {
                        PrintAPIResult(mAPIResult);
                    }

                    
                }

                // upis u datoteku             
                string mDataFileName = "GETBAZE-data." + mAPIResult.database_format + "." + mAPIResult.GetDatabaseFileExtension();
                Console.WriteLine("Podatci snimljeni u: " + mDataFileName);
                mAPIResult.SaveDataBaseAttachmentToFile(mDataFileName);
            }
            else
            {
                Console.WriteLine("Greska:\r\n" + mAPIResponse.error_state + "\r\n" + mAPIResponse.error_message);
            }

        }

        public InfoAPIResponse ExecuteAPI(InfoAPIParams pAPIParams)
        {
            this.APIParams = pAPIParams;

            this.PrepareAPICall();

            // IZVRSI POZIV API FUNKCIJE
            // u mAPIResponse se nalazi odgovor API-ja. U mAPIResponse.Result je rezultat izvrsenja funkcije.
            InfoAPIResponse mAPIResponse = null;

            mAPIResponse = APIHandler.CallAPI(pAPIParams);

            return mAPIResponse;
        }

        private void PrepareAPICall()
        {
            // podešavanje hendlera API konekcije
            this.APIHandler = new InfoAPIHandler();
            this.APIHandler.APIServerURL = "http://apitest.infosys.rs:31000/InfoAPI.aspx";
            this.APIHandler.username = "info";
            this.APIHandler.password = "demo";
            this.APIHandler.client_id = "DEMO";
            this.APIHandler.client_pk = "87oluavs978bl94vkivfzs967zs3rf";
            this.APIHandler.client_version = "0.0.1";
            this.APIHandler.SerializeIdented = true;
            this.APIParams.result_database_format = this.TableFormat;

            // dodaj Class factory za modul DEMO
            this.APIHandler.InfoAPIClassFactories.Add(new GETBAZEClassFactory());
        }

        
        public void PrintAPIResult(InfoAPIResultGETBAZE pResult)
        {
            Console.WriteLine("Generic data\n");
            Console.WriteLine("Error state: " + pResult.error_state);
            Console.WriteLine("Error message: " + pResult.error_message);
            Console.WriteLine("Exec date time: " + pResult.exec_date_time);
            Console.WriteLine("\nResponse Data\n");

            this.PrintTableKomitenti(Repository.Komitenti);
            this.PrintTableMestaIsporuke(Repository.MestaIsporuke);
        }

        public void PrintTableKomitenti(cKomitenti pTable)
        {
            if (pTable != null)
            {
                Console.WriteLine("Tabela:");
                int pBrojac = 1;
                string formatIspisa = "{0,4} {1,4}  {2,-30} {3,-10} {4,-10} {5,-10} {6,-25}";
                Console.WriteLine(formatIspisa,
                    "RB",
                    "ID",
                    "Naziv",
                    "Mesto",
                    "Broj Poste",
                    "Ulica",
                    "Broj"
                    );
                foreach (var item in pTable)
                {
                    Console.WriteLine(formatIspisa,
                    pBrojac,
                    item.id_komitenta,
                    item.naziv,
                    item.mesto,
                    item.br_poste,
                    item.ulica,
                    item.broj);
                    pBrojac++;
                }
            }

        }

        public void PrintTableMestaIsporuke(cMestaIsporuke pTable)
        {
            if (pTable != null)
            {
                Console.WriteLine("Tabela:");
                int pBrojac = 1;
                string formatIspisa = "{0,4} {1,4}  {2,-30} {3,-10} {4,-10} {5,-10} {6,-25}";
                Console.WriteLine(formatIspisa,
                    "RB",
                    "ID",
                    "ID Komitenta",
                    "Naziv",
                    "Mesto",
                    "Ulica",
                    "Broj"
                    );
                foreach (var item in pTable)
                {
                    Console.WriteLine(formatIspisa,
                    pBrojac,
                    item.id_mesta_isporuke,
                    item.id_komitenta,
                    item.naziv,
                    item.mesto,
                    item.ulica,
                    item.broj);
                    pBrojac++;
                }
            }

        }

    }
}
