﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InfoAPILib;
using System.Diagnostics;

namespace InfoAPIDemoMATH
{
    class Program
    {
        private InfoAPIHandler APIHandler;
        private InfoAPIParams APIParams;


        static void Main(string[] args)
        {
            Stopwatch sw = new Stopwatch();
            Program program = new Program();
            string lFunction, lParam1, lParam2;

            PrepareInput(out lFunction, out lParam1, out lParam2);

            sw.Start();
            program.RunTest(lFunction, lParam1, lParam2);
            Console.WriteLine("\nBenchmark: " + sw.Elapsed.TotalSeconds + " SEC");
            Console.ReadLine();
        }

        private static void PrepareInput(out string lFunction, out string lParam1, out string lParam2)
        {
            do
            {
                Console.WriteLine("Odaberite operaciju koju zelite:\n1. ADD\n2. SUB\n3. DIV\n4. MUL\n");

                lFunction = Console.ReadLine();
                lFunction = lFunction.ToLower();
                switch (lFunction)
                {
                    case "1":
                    case "add":
                        lFunction = "ADD";
                        break;
                    case "2":
                    case "sub":
                        lFunction = "SUB";
                        break;
                    case "3":
                    case "div":
                        lFunction = "DIV";
                        break;
                    case "4":
                    case "MUL":
                        lFunction = "MUL";
                        break;
                    default:
                        lFunction = "";
                        break;
                }
            } while (lFunction == "");

            float lParam1Float;
            float lParam2Float;

            do
            {
                Console.Write("Unesite prvi broj: ");
                lParam1 = Console.ReadLine();
            } while (!float.TryParse(lParam1, out lParam1Float));

            do
            {
                Console.Write("Unesite drugi broj: ");
                lParam2 = Console.ReadLine();
            } while (!float.TryParse(lParam2, out lParam2Float));
        }

        public void RunTest(string pFunction, string pParam1, string pParam2)
        {
            InfoAPIParamsMATH mAPIParams = new InfoAPIParamsMATH();
            mAPIParams.function = pFunction;
            mAPIParams.param1 = float.Parse(pParam1);
            mAPIParams.param2 = float.Parse(pParam2);

            //InfoAPIResultMATH mAPIResult = new InfoAPIResultMATH();

            InfoAPIResponse mResponse = this.ExecuteAPI(mAPIParams);

            if (mResponse != null)
            {
                if (mResponse.error_state == "ok")
                {
                    PrintAPIResult((InfoAPIResultMATH)mResponse.Result);
                }
                else
                {
                    Console.WriteLine("Greska:\r\n" + mResponse.error_state + "\r\n" + mResponse.error_message);
                }
            } else
            {
                Console.WriteLine("API call error!");
            }
        }


        public InfoAPIResponse ExecuteAPI(InfoAPIParams pAPIParams)
        {
            this.APIParams = pAPIParams;

            this.PrepareAPICall();

            // IZVRSI POZIV API FUNKCIJE
            // u mAPIResponse se nalazi odgovor API-ja. U mAPIResponse.Result je reyultat izvrsenja funkcije.
            InfoAPIResponse mAPIResponse = null;

            //try
            //{
            // mAPIResponse = APIHandler.CallAPI(pAPIParams, pAPIResult);
               mAPIResponse = APIHandler.CallAPI(pAPIParams);
            //}
            //catch (InfoAPICallException e)
            //{
            //    Console.WriteLine("Greska:\r\n" + e.ErrorInfo.error_state + "\r\n" + e.ErrorInfo.error_message);
            //    Console.WriteLine("API call error!");
            //}
            return mAPIResponse;
        }


        private void PrepareAPICall()
        {
            // podešavanje hendlera API konekcije
            this.APIHandler = new InfoAPIHandler();
            this.APIHandler.APIServerURL = "http://apitest.infosys.rs:31000/InfoAPI.aspx";
            this.APIHandler.username = "info";
            this.APIHandler.password = "demo";
            this.APIHandler.client_id = "DEMO";
            this.APIHandler.client_pk = "87oluavs978bl94vkivfzs967zs3rf";
            this.APIHandler.client_version = "0.0.1";
            this.APIHandler.SerializeIdented = true;
            this.APIParams.result_database_format = "PLAIN_OBJ";

            // dodaj Class factory za modul DEMO
            this.APIHandler.InfoAPIClassFactories.Add (new MATHClassFactory());
        }


        public void PrintAPIResult(InfoAPIResultMATH pResult)
        {
            Console.WriteLine("Generic data\n");
            Console.WriteLine("Error state: " + pResult.error_state);
            Console.WriteLine("Error message: " + pResult.error_message);
            Console.WriteLine("Exec date time: " + pResult.exec_date_time);
            Console.WriteLine("\nResponse Data\n");
            Console.WriteLine("Value: " + pResult.value);
        }

        
    }
}
