﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InfoAPILib;
using System.Diagnostics;
using System.Collections;

namespace InfoAPIDemoSENDTABLE
{
    class Program
    {
        private InfoAPIHandler APIHandler;
        private InfoAPIParams APIParams;
        private ExampleTable TableData = null;
        private string TableFormat;

        static void Main(string[] args)
        {
            Stopwatch sw = new Stopwatch();
            Program program = new Program();

            program.PrepareInput(out program.TableFormat);

            sw.Start();
            program.RunTest();
            Console.WriteLine("\nBenchmark: " + sw.Elapsed.TotalSeconds + " SEC");
            Console.ReadLine();
        }

        public void PrepareInput(out string pFormat)
        {
            do
            {
                Console.WriteLine("Odaberite format:\n1. PLAIN_OBJ\n2. SQLLITE\n3. ZIPDBF\n");

                pFormat = Console.ReadLine();
                pFormat = pFormat.ToLower();
                switch (pFormat)
                {
                    case "1":
                    case "plain_obj":
                        pFormat = "PLAIN_OBJ";
                        break;
                    case "2":
                    case "sqllite":
                        pFormat = "SQLITE";
                        break;
                    case "3":
                    case "zipdf":
                        pFormat = "ZIPDBF";
                        break;
                    default:
                        pFormat = "";
                        break;
                }
            } while (pFormat == "");
        }

        public void RunTest()
        {
            //Poziv api funkcije
            InfoAPIParamsSENDTABLE mAPIParams = new InfoAPIParamsSENDTABLE();
            mAPIParams.database_format = TableFormat;
            switch (mAPIParams.database_format)
            {
                case "PLAIN_OBJ":

                    // kreiraj primer tabelarnih podataka (bilo sta sto ima IEnumerable interfejs)
                    // u ovom primaru je ExampleTable naslednik TList
                    InfoAPIDemoSENDTABLE.ExampleTable lExampleTable = new InfoAPIDemoSENDTABLE.ExampleTable();

                    // generisi primer podataka u listi
                    lExampleTable.Populate();

                    mAPIParams.AddTableReference("tabela", lExampleTable);

                    break;

                case "SQLITE":
                    mAPIParams.LoadDatabaseAttachmentFromFile("SENDTABLE-data.sqlite");
                    break;

                case "ZIPDBF":
                    mAPIParams.LoadDatabaseAttachmentFromFile("SENDTABLE-data.ZIPDBF.zip");
                    break;

            }
            InfoAPIResponse mAPIResponse = this.ExecuteAPI(mAPIParams);
            
            
            if (mAPIResponse != null)
            {
                InfoAPIResultSENDTABLE mAPIResult = (InfoAPIResultSENDTABLE) mAPIResponse.Result;

                if (mAPIResponse.error_state.ToUpper() == InfoAPIResponse.ERROR_STATE_OK.ToUpper())
                {
                    
                   

                }
                else
                {
                    Console.WriteLine("Greska:\r\n" + mAPIResponse.error_state + "\r\n" + mAPIResponse.error_message);
                }

                if (mAPIResponse.error_state == "ok")
                {
                    PrintAPIResult((InfoAPIResultSENDTABLE) mAPIResponse.Result);
                }
                else
                {
                    Console.WriteLine("Greska:\r\n" + mAPIResponse.error_state + "\r\n" + mAPIResponse.error_message);
                }
            }
            else
            {
                Console.WriteLine("API call error!");
            }
        }


        public InfoAPIResponse ExecuteAPI(InfoAPIParams pAPIParams)
        {
            this.APIParams = pAPIParams;

            this.PrepareAPICall();

            // IZVRSI POZIV API FUNKCIJE
            // u mAPIResponse se nalazi odgovor API-ja. U mAPIResponse.Result je rezultat izvrsenja funkcije.
            InfoAPIResponse mAPIResponse = null;

            mAPIResponse = APIHandler.CallAPI(pAPIParams);

            return mAPIResponse;
        }


        private void PrepareAPICall()
        {
            // podešavanje hendlera API konekcije
            this.APIHandler = new InfoAPIHandler();
            //this.APIHandler.APIServerURL = "http://pedjas.infosys.rs:31000/InfoAPI.aspx";
            this.APIHandler.APIServerURL = "http://apitest.infosys.rs:31000/InfoAPI.aspx";
            this.APIHandler.username = "info";
            this.APIHandler.password = "demo";
            this.APIHandler.client_id = "DEMO";
            this.APIHandler.client_pk = "87oluavs978bl94vkivfzs967zs3rf";
            this.APIHandler.client_version = "0.0.1";
            this.APIHandler.SerializeIdented = true;
            this.APIParams.result_database_format = this.TableFormat;

            // dodaj Class factory za modul DEMO
            this.APIHandler.InfoAPIClassFactories.Add(new SENDTABLEClassFactory());
        }


        public void PrintAPIResult(InfoAPIResultSENDTABLE pResult)
        {


            Console.WriteLine("Generic data\n");
            Console.WriteLine("Error state: " + pResult.error_state);
            Console.WriteLine("Error message: " + pResult.error_message);
            Console.WriteLine("Exec date time: " + pResult.exec_date_time);
            Console.WriteLine("\nResponse Data\n");

            PrintTable(this.TableData);

        }

        public void PrintTable(ExampleTable pTable)
        {
            if (pTable != null)
            {
                Console.WriteLine("Tabela:");
                int pBrojac = 1;
                string formatIspisa = "{0,4} {1,4}  {2,-30} {3,-10} {4,-10} {5,-10} {6,-25}";
                Console.WriteLine(formatIspisa,
                    "RB",
                    "ID",
                    "Opis",
                    "Grupa",
                    "Mesto",
                    "Oznaka",
                    "Datum"
                    );
                foreach (var item in pTable)
                {
                    Console.WriteLine(formatIspisa,
                    pBrojac,
                    item.id,
                    item.opis,
                    item.grupa,
                    item.mesto,
                    item.oznaka,
                    item.datum_i_vreme);
                    pBrojac++;
                }
            }

        }
    }
}
