﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization;
using System.Dynamic;

namespace InfoAPILib
{
    /// <summary>
    /// InfoAPIBase je bazna klasa za InfoAPIProperties i InfoAPIEnvelope. Sadrži zajedničku
    /// funkcionalnost svih klasa koje čine API poruku, Pre svega, obezbeđuje serijalizaciju i deserijalizaciju.
    /// </summary>
    [DataContract]
    public class InfoAPIBase// : DynamicObject
    {

        /// <summary>
        /// Конструктор
        /// </summary>
        public InfoAPIBase ()
        {
        }

        internal string __class_name = "";

        /// <summary>
        /// Naziv klase. U API porukama je ovo uglavnom informativnog karaktera, a dobro dođe 
        /// kada se analiziraju složene strukture
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string _class_name
        {
            get
            {
                if (string.IsNullOrEmpty(this.__class_name)) {
                    return GetType().Name;
                } else  {
                    return this.__class_name;
                }
            }
            internal set
            {
                this.__class_name = value;
            }
        }

        
        private string _app_option = null;
        /// <summary>
        /// app_option(char) 
        /// Oznaka API funcije koju klijent poziva. Ovo je podatak na osnovu koga API server zna koju funkciju treba da izvrši.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string app_option {
            get {
                    return this._app_option;
            }

            set {
                if (value != null)
                {
                    this._app_option = value;
                } else  {
                    throw new ArgumentNullException();
                }
            }
        }

        /// <summary>
        /// Objekat koji vrši serijalizaciju i deserijalizaciju
        /// </summary>
        public IInfoAPIFormatter Formatter;  // formater ce da vrsi deserijalizaciju

        /// <summary>
        /// Serijalizuje objekat
        /// </summary>
        /// <returns>String koji sadrži serijalizovani objekat</returns>
        public string Serialize()
        {
            return this.Serialize(false);
        }
        /// <summary>
        /// Serializuje objekat sa identacijom
        /// </summary>
        /// <param name="pIndented">Da li se uključuje i identacija u serijalizaciju</param>
        /// <returns>String koji sadrži serijalizovani objekat</returns>
        public string Serialize(bool pIndented)
        {
            return this.Formatter.Serialize(this); //, pIndented);
        }
        /// <summary>
        /// Deserijalizuje string u objekat
        /// </summary>
        /// <param name="pSerializedInput">String koji terba deserijalizovati</param>
        public void Deserialize(string pSerializedInput)
        {
            this.Formatter.Deserialize(pSerializedInput, this);
        }


/*

        // If you try to get a value of a property 
        // not defined in the class, this method is called.
        public override bool TryGetMember(
            GetMemberBinder binder, out object result)
        {
            // Converting the property name to lowercase
            // so that property names become case-insensitive.
            string name = binder.Name.ToLower();

            // If the property name is found in a dictionary,
            // set the result parameter to the property value and return true.
            // Otherwise, return false.
            //return dictionary.TryGetValue(name, out result);
            result = null;
            return false;
        }

        // If you try to set a value of a property that is
        // not defined in the class, this method is called.
        public override bool TrySetMember(
            SetMemberBinder binder, object value)
        {
            // Converting the property name to lowercase
            // so that property names become case-insensitive.
            //dictionary[binder.Name.ToLower()] = value;

            // You can always add a value to a dictionary,
            // so this method always returns true.
            return true;
        }
*/

    }

}
