﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace InfoAPILib
{

    public class InfoAPIDynamicTable : List<dynamic>
    {

        public InfoAPITableFields fields = null;
        //public InfoAPITableRows rows;

        public InfoAPIDynamicTable()
        {
            //this.rows = new InfoAPITableRows();
        }

        public void SetFields(string[] pFields)
        {
            this.fields = new InfoAPITableFields(pFields);

        }

        private new void Add(object pRowData)
        {
            base.Add(pRowData);
        }

        private void Add(string[] pRowData)
        {
            if (this.fields != null)
            {
                dynamic lRow = new ExpandoObject();
                var lRowDictionary = (IDictionary<string, object>)lRow;

                int lFieldCount = 0;
                foreach (KeyValuePair<string, InfoAPITableField> lFieldData in this.fields)
                {
                    if (pRowData != null) { 
                        object lValue = this.ConvertType(lFieldData.Value.Type, pRowData[lFieldCount]);
                        if (lFieldData.Value.Type == this.MapType(lValue))
                        {
                            lRowDictionary.Add(lFieldData.Key, lValue);
                        }
                        else
                        {
                            throw new Exception("Field " + lFieldData.Key + " type is " + lFieldData.Value.Type + ". " + lValue.GetType() + " cannot be assigned.");
                        }
                    }
                    else
                    {
                        lRow.Add(lFieldData.Key, null);
                    }
                    lFieldCount++;
                }
                
                this.Add(lRow);
            } else
            {
                throw new Exception("You must set fields in InfoAPIDynamicTable before adding rows as string array!");
            }
        }

        public void AddRow(string[] pRowData)
        {
            this.Add(pRowData);

        }


        public string MapType(object pValue)
        {
            string lResult = "U";

            Type lType = pValue.GetType();
            switch (lType.Name)
            {
                case "String":
                    lResult = "C";
                    break;
                case "Boolean":
                    lResult = "L";
                    break;


                case "Int32":
                case "UInt32":
                case "Int64":
                case "UInt64":
                case "Int16":
                case "UInt16":
                case "Byte":
                case "SByte":
                    lResult = "I";
                    break;

                case "Double":
                case "Decimal":
                case "Single":
                case "Float":
                    lResult = "N";
                    break;

                case "DateTime":
                case "Date":
                case "Time":
                    lResult = "D";
                    break;

                default:
                    lResult = "U";
                    break;
            }
            return lResult;
        }

        public object ConvertType(string pType, object pValue)
        {
            object lResult;


            switch (pType)
            {
                case "C":
                    lResult = Convert.ToString(pValue);
                    break;
                case "L":
                    lResult = Convert.ToBoolean(pValue);
                    break;
                case "N":
                    lResult = Convert.ToDouble(pValue);
                    break;

                case "D":
                    lResult = Convert.ToDateTime(pValue);
                    break;

                default:
                    throw new Exception("Cannot convert value to field type " + pType + "!");
                    break;
            }


            return lResult;
        }
    }

}
