﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Collections.Specialized;
using System.Reflection;
using RestSharp;
using Newtonsoft.Json;

namespace InfoAPILib
{

    #region Event InfoAPICallCompleted

    /// <summary>
    /// Tip argumenta događaja InfoAPICallCompleted
    /// </summary>
    public class InfoAPICallCompletedEventArgs : EventArgs
    {
        /// <summary>
        /// Odgovor od API-ja
        /// </summary>
        public InfoAPIResponse InfoAPIResponse { get; private set; }
        /// <summary>
        /// Parametri poslati API funkciji
        /// </summary>
        public InfoAPIParams InfoAPIParams { get; private set; }

        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="pInfoAPIResponse"></param>
        public InfoAPICallCompletedEventArgs (InfoAPIResponse pInfoAPIResponse)
        {
            this.InfoAPIResponse = pInfoAPIResponse;
        }
    }

    /// <summary>
    /// Događaj InfoAPICallCompletedEventHandler se pokreće kada se završi poziv API funkciji
    /// </summary>
    /// <param name="sender">Objekat koji aktivira događaj</param>
    /// <param name="e">Podaci koji se šalju hendleru.</param>
    public delegate void InfoAPICallCompletedEventHandler (object sender, InfoAPICallCompletedEventArgs e);
    #endregion

    #region Event InfoAPICallError
    /// <summary>
    /// Tip argumenta događaja InfoAPICallError
    /// </summary>
    public class InfoAPICallErrorEventArgs : EventArgs
    {
        public string error_state { set; get; }
        public string error_message { set; get; }
        public object error_data { set; get; }
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender">Objekat koji aktivira događaj</param>
    /// <param name="e">Podaci koji se šalju hendleru.</param>
    public delegate void InfoAPICallErrorEventHandler(object sender, InfoAPICallErrorEventArgs e);
    #endregion

    #region Event InfoAPICallBeforeCall
    /// <summary>
    /// Tip argumenta događaja InfoAPICallBeforeCall
    /// </summary>
    public class InfoAPICallBeforeCallEventArgs : EventArgs
    {
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender">Objekat koji aktivira događaj</param>
    /// <param name="e">Podaci koji se šalju hendleru.</param>
    public delegate void InfoAPICallBeforeCallEventHandler(object sender, InfoAPICallBeforeCallEventArgs e);
    #endregion

    #region Event InfoAPICallAfterCall
    /// <summary>
    /// Tip argumenta događaja APICallAfterCall
    /// </summary>
    public class InfoAPICallAfterCallEventArgs : EventArgs
    {
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender">Objekat koji aktivira događaj</param>
    /// <param name="e">Podaci koji se šalju hendleru.</param>
    public delegate void InfoAPICallAfterCallEventHandler(object sender, InfoAPICallAfterCallEventArgs e);
    #endregion

    /// <summary>
    /// Definicija InfoAPICallException Exception-a
    /// </summary>
    public class InfoAPICallException : Exception
    {
        public InfoAPICallErrorEventArgs ErrorInfo;
        public InfoAPICallException(InfoAPICallErrorEventArgs e) : base(e.error_message) {
            this.ErrorInfo = e;
        }
    }


    /// <summary>
    /// InfoAPIHandler hendluje poziv APi funkcija. Korisnik samo treba da napravi objekte naslednike InfoAPIParams i 
    /// InfoAPIResponse koje prosledi ovom hendleru a on obavlja ceo posao API komunikacije.
    /// </summary>
    public class InfoAPIHandler
    {
        /// <summary>
        /// APIServerURL - adresa API servera
        /// </summary>
        public string APIServerURL { get; set; }

        /// <summary>
        /// session_id
        /// Идентификатор сесије. Овај идентификатор генерише сервер када
        /// клијент први пут позове АПИ и враћа га клијенту.Клијент у сваком
        /// следећем упиту у оквиру исте сесије шаље идентификатор сесије.
        /// Ово омогућава да се одређени подаци или стања могу преносити
        /// између више упита са истог клијента ка АПИ-ју.
        /// Сесија се прекида одјавом или неактивношћу клијента.Подразумевано,
        /// некативност дужа од 10 минута прекида сесију, а то се може
        /// подешавати на АПИ северу.
        /// Ако клијент не зна идентификатор сесије, односно тек започиње сесију,
        /// шаље ово поље празно.
        /// Ако клијетн жели да прекине текућу сесију и започне нову, треба да
        /// session_id подеси да иам вреднсот празног стринга. У том случају 
        /// ће се у упиту послати username и password да се започне нова сесија.
        /// </summary>
        public string session_id { get; set; }

        /// <summary>
        /// instance_id (char) 
        /// Идентификатор инстанце у оквиру сесије. Овај идентификатор сервер
        /// генерише ако није добио идентификатор од клијента.Клијент по
        /// правилу у сваком наредном упиту шаље идентификатор инстанце који
        /// је добио од сервера осим ако не жели да започне нову инстанцу.
        /// Клијент може да покрене више инстанци у оквиру исте сесије. Све
        /// инстанце се затварају са затварањем сесије.        
        public string instance_id { get; set; }

        /// <summary>
        /// encoding (char) 
        /// Начин на који су кодирани подаци у пољу data. Сервер на основу овог
        /// податка зна како да декодира податке.Подразумевано се подаци не
        /// кодирају и ово поље остаје празно.Енкодирање података није још увек
        /// имплементирано.
        /// </summary>
        public string encoding { get; set; }

        /// <summary>
        /// Пословна година на коју се односу упит. Ако ниеј наведена, АПИ користи
        /// </summary>
        public string poslovna_godina { get; set; }
        /// <summary>
        /// username (char) 
        /// Корисничко име за пријаву корисника АПИ серверу. Употребљава се
        /// заједно са пољем password. Користи се приликом креирања нове сесије
        /// да би се провериле привилегије и права приступа наведеног корисника.
        /// </summary>
        public string username { get; set; }
        
        /// <summary>
        /// password (char) 
        /// Лозинка која се употребљава за пријаву корисника, заједно са username.
        /// </summary>
        public string password { get; set; }

        /// <summary>
        /// client_id (char) 
        /// Ознака клијентског програма (везано за лиценцу)
        /// </summary>
        public string client_id { get; set; }

        /// <summary>
        /// client_version (char) 
        /// Ознака верзије клијентског програма
        /// </summary>
        public string client_version { get; set; }

        /// <summary>
        /// client_pk (char) 
        /// Приватни кључ клијентског програма (везано за лиценцу)
        /// </summary>
        public string client_pk { get; set; }


        /// <summary>
        /// Nivo debug informacija koju server treba da pošalje klijentu
        /// </summary>
        public int DebugLevel { get; set; }
        /// <summary>
        /// Debug sekcije koje treba vratiti klijentu
        /// </summary>
        public string DebugSections { get; set; }

        /// <summary>
        /// Da li ce serijalizovani string biti formatiran da bude citak
        /// </summary>
        public bool SerializeIdented = false;




        public string QueryString;
        public string ParamsString;
        public string ResponseString;

        /// <summary>
        /// Podešavanje da li će APILIb da prijavi izuzetak ako se dogodi greška ili ne. Podrazumevano se izuzetak ne prijavljuje.
        /// </summary>
        public Boolean TriggerInfoAPICallException = false;

        /// <summary>
        /// Lista fabrika klasa InfoAPIParams i InfoAPIResult. Za svaki modul sa API funkcijama treba da postoji projekat koji 
        /// sadrži biblioteke klasa InfoAPIParams i InfoAPIResult za svaku finkciju i jodenog naslednika InfoAPIClassFactory koji 
        /// će umeti da po imenu API funkcije napravi sve te klase kada mu ih neko zatraži.
        /// </summary>
        public List<IInfoAPIClassFactory> InfoAPIClassFactories;


        public InfoAPIHandler()
        {
            this.DebugLevel = 0;
            this.DebugSections = "";
            this.InfoAPIClassFactories = new List<IInfoAPIClassFactory>();
            this.InfoAPIClassFactories.Add(new InfoAPIClassFactory());
        }

        public InfoAPIResult GetInfoAPIResultClass(string pAppOption) {

            int lCount = this.InfoAPIClassFactories.Count();
            int lCounter = 0;

            InfoAPIResult lResult;
            lResult = null;

            while ( (lResult == null) && (lCounter < (lCount)))
            {
                IInfoAPIClassFactory lFactory = this.InfoAPIClassFactories[lCounter];
                lResult = lFactory.GetResultObject(pAppOption);
                lCounter++;
            };

            if (lResult == null) {
                lResult = new InfoAPIResultUniversal();
                lResult.app_option = pAppOption;
                //throw new Exception("InfoAPIParams class for API function " + pAppOption + " not found or not accessible!");
            }

            return lResult;
        }

        /// <summary>
        /// Event handler koji se poziva kada je poziv API funkcije izvršen
        /// </summary>
        public event InfoAPICallCompletedEventHandler APICallCompleted;

        /// <summary>
        /// Događaj koji se pokreće kada je završen poziv API funkciji
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected virtual void onAPICallCompleted(object sender, InfoAPICallCompletedEventArgs e)
        {
            this.APICallCompleted?.Invoke(this, e);
        }


        /// <summary>
        /// Event handler koji se poziva kada je nastala greska prilikom poziva API funkcije
        /// </summary>
        public event InfoAPICallErrorEventHandler APICallError;

        protected virtual void onAPICallError(object sender, InfoAPICallErrorEventArgs e)
        {
            /*
            if (this.APICallError != null) {
                this.APICallError.Invoke(this, e);
            } else
            {
                if (this.TriggerInfoAPICallException) { 
                    throw new InfoAPICallException(e);
                }
            }
            */

            Boolean mTrigerException = true; // this.TriggerInfoAPICallException;

            if (this.APICallError != null)
            {
                this.APICallError.Invoke(this, e);
            }
            else
            {
                //mTrigerException = this.TriggerInfoAPICallException;
                mTrigerException = true;
            }

            if (mTrigerException)
            {
               throw new InfoAPICallException(e);
            }

        }


        /// <summary>
        /// Event handler koji se poziva pre poziva API funkcije
        /// </summary>
        public event InfoAPICallBeforeCallEventHandler APICallBeforeCall;

        protected virtual void onAPICallBeforeCall(object sender, InfoAPICallBeforeCallEventArgs e)
        {
            this.APICallBeforeCall?.Invoke(this, e);
        }

        /// <summary>
        /// Event handler koji se poziva posle poziva API funkcije
        /// </summary>
        public event InfoAPICallAfterCallEventHandler APICallAfterCall;

        protected virtual void onAPICallAfterCall(object sender, InfoAPICallAfterCallEventArgs e)
        {
            this.APICallAfterCall?.Invoke(this, e);
        }


        /// <summary>
        /// Pripremi upit API funkciji. Ulaz su samo parametri koji se šalju funkciji (sadrže i naziv API funkcije).
        /// </summary>
        /// <param name="pAPIParams">Parametri API funkciji</param>
        /// <returns>Objekat koji sadrži pripremljen kompletan upit API funkciji</returns>

        InfoAPIRequest PrepareRequest(InfoAPIParams pAPIParams)
        {

            InfoAPIFormatterJSON APIFormatterJSON = new InfoAPIFormatterJSON();
            if (this.SerializeIdented)
            {
                APIFormatterJSON.Formatting = Formatting.Indented;
            }

            pAPIParams.Formatter = APIFormatterJSON;


            InfoAPIRequest mAPIRequest = new InfoAPIRequest();
            mAPIRequest.Formatter = APIFormatterJSON;

            // da bi se lakse citao JSON u debugingu formatiraj ga. Inace ne treba ovo raditi.
            //APIFormatterJSON.Formatting = Newtonsoft.Json.Formatting.Indented;

            if (String.IsNullOrWhiteSpace(this.session_id))
            {
                mAPIRequest.username = this.username;
                mAPIRequest.password = this.password;
            }
            else
            {
                mAPIRequest.session_id = this.session_id;
            }


            mAPIRequest.app_option = pAPIParams.app_option;
            mAPIRequest.instance_id = this.instance_id;
            mAPIRequest.client_id = this.client_id;
            mAPIRequest.client_version = this.client_version;
            mAPIRequest.encoding = this.encoding;
            mAPIRequest.poslovna_godina = this.poslovna_godina;

            pAPIParams.RequestEnvelope = mAPIRequest;

            mAPIRequest.data = pAPIParams.Serialize();
            this.ParamsString = mAPIRequest.data;

            mAPIRequest.SetChecksum(this.client_pk);

            this.QueryString = mAPIRequest.Serialize();
            return mAPIRequest;

        }

        /// <summary>
        /// Na osnovu InfoAPIResult pripremi objekat za odgovor (InfoAPIResponse). 
        /// Ovaj objekat se može upotrebiti za prijem rezultata izvrsenja API funkcije.
        /// </summary>
        /// <param name="pAPIResult"></param>
        /// <returns></returns>
        InfoAPIResponse PrepareResponse(string pAppOption)
        {

            InfoAPIResult mAPIResult = (InfoAPIResult)this.GetInfoAPIResult(pAppOption);

            InfoAPIFormatterJSON APIFormatterJSON = new InfoAPIFormatterJSON();

            if (this.SerializeIdented) { 
                APIFormatterJSON.Formatting = Formatting.Indented;
            }

            InfoAPIResponse mAPIResponse = new InfoAPIResponse();
            mAPIResponse.Formatter = APIFormatterJSON;
            mAPIResponse.Result = mAPIResult;
            mAPIResponse.Result.Formatter = APIFormatterJSON;
            return mAPIResponse;
        }

        /// <summary>
        /// Vrati listu koja predstavlja POST parametre za HTTP upit koji se šalje InfoAPI serveru.
        /// </summary>
        /// <returns>Kolekcija koja sadrži vrednosti POST parametera</returns>
        NameValueCollection GetPostVars()
        {
            NameValueCollection POSTVars = new NameValueCollection();
            POSTVars["json"] = this.QueryString;
            POSTVars["DebugLevel"] = this.DebugLevel.ToString();
            POSTVars["DebugSection"] = this.DebugSections;
            return POSTVars;
        }

        private InfoAPIResult GetInfoAPIResult (string pAppOption)
        {
            //string lParamsClassName = pInfoAPiParams.GetType().ToString();
            //string lResultClassName = lParamsClassName.Replace("InfoAPIParams", "InfoAPIResult");
            //string lClassName = "InfoAPILib.InfoAPIResult" + lParamsClassName; // pAppOption.ToUpper();
            //InfoAPIResult lInfoAPIResult = (InfoAPIResult)System.Reflection.Assembly.GetExecutingAssembly().CreateInstance(lResultClassName);
            InfoAPIResult lInfoAPIResult = (InfoAPIResult)this.GetInfoAPIResultClass(pAppOption);
            return lInfoAPIResult;
        }

        /// <summary>
        /// Pozovi funkciju na API serveru sa parametrima navedenim u pAPIParams
        /// </summary>
        /// <param name="pAPIParams">Parametri koji se šalju API funkciji</param>
        /// <param name="pAPIResult">Rezultat API funkcije. Daje se unapred pripremljen objekat koji će biti napunjen rezultatom</param>
        /// <returns>Odgovor od InfoAPI servera</returns>
        //public InfoAPIResponse CallAPI(InfoAPIParams pAPIParams, InfoAPIResult pAPIResult)
        public InfoAPIResponse CallAPI(InfoAPIParams pAPIParams)
        {

            InfoAPIRequest mAPIRequest = this.PrepareRequest(pAPIParams);

            //InfoAPIResult mAPIResult = (InfoAPIResult)this.GetInfoAPIResult(pAPIParams.app_option);

            
            //InfoAPIResponse mAPIResponse = this.PrepareResponse(pAPIParams.app_option);
            InfoAPIResponse mAPIResponse = null;

            this.onAPICallBeforeCall(this, new InfoAPICallBeforeCallEventArgs());

            WebClient mWebClient = new WebClient();
 

            try
            {

                var POSTVars = this.GetPostVars();

                var client = new RestClient(this.APIServerURL);
                var request = new RestRequest("", Method.POST);
                request.AddParameter("json", this.QueryString);
                request.AddParameter("DebugLevel", this.DebugLevel.ToString());
                request.AddParameter("DebugSection", this.DebugSections);
                IRestResponse response = client.Execute(request);

                if (response.ErrorException != null)
                {
                    throw response.ErrorException;
                }

                this.ResponseString = response.Content; // raw content as string

                //var mResponse = mWebClient.UploadValues(this.APIServerURL, "POST", POSTVars);
                //this.ResponseString = Encoding.Default.GetString(mResponse);

                mAPIResponse = this.PrepareResponse(pAPIParams.app_option);
                mAPIResponse.Deserialize(this.ResponseString);

                if (!string.IsNullOrEmpty(mAPIResponse.app_option))
                {
                    
                    mAPIResponse.Result.app_option = mAPIResponse.app_option;
                    //pAPIResult.app_option = mAPIResponse.app_option;
                }

                if (mAPIResponse.isResultStateOK())
                {
                    mAPIResponse.Result.Deserialize(mAPIResponse.data);

                    InfoAPICallCompletedEventArgs mAPICallCompletedEventArgs = new InfoAPICallCompletedEventArgs(mAPIResponse);
                    this.onAPICallCompleted(this, mAPICallCompletedEventArgs);
                } else {
                    /*
                     * 
                     * iskljuceno jer ako je funkciaj vratila gresku to je regularno, ne treba exception
                     * 
                    InfoAPICallErrorEventArgs mAPICallErrorEventArgs = new InfoAPICallErrorEventArgs();
                    mAPICallErrorEventArgs.error_message = mAPIResponse.error_message;
                    mAPICallErrorEventArgs.error_state = mAPIResponse.error_state;
                    mAPICallErrorEventArgs.error_data = mAPIResponse.Result.error_data;

                    this.onAPICallError(this, mAPICallErrorEventArgs);
                    */
                }



            } catch (Exception e)
            {
                InfoAPICallErrorEventArgs mAPICallErrorEventArgs = new InfoAPICallErrorEventArgs();
                mAPICallErrorEventArgs.error_state = "API_CALL_ERROR";
                mAPICallErrorEventArgs.error_message = e.Message;

                this.onAPICallError(this, mAPICallErrorEventArgs);
            }
            finally
            {
                this.onAPICallAfterCall(this, new InfoAPICallAfterCallEventArgs());
            }

            return mAPIResponse;
        }

        /// <summary>
        /// Pozovi funkciju asinhrono na API serveru sa parametrima navedenim u pAPIParams
        /// </summary>
        /// <param name="pAPIParams">Parametri koji se šalju API funkciji</param>
        /// <param name="pAPIResult">Rezultat API funkcije. Daje se unapred pripremljen objekat koji će biti napunjen rezultatom</param>
        //public async void CallAPIAsync(InfoAPIParams pAPIParams, InfoAPIResult pAPIResult)
        public async void CallAPIAsync(InfoAPIParams pAPIParams)
        {
            //await  _CallAPIAsync(pAPIParams,pAPIResult);
            await _CallAPIAsync(pAPIParams);
        }

        //private async Task _CallAPIAsync(InfoAPIParams pAPIParams, InfoAPIResult pAPIResult)
        private async Task _CallAPIAsync(InfoAPIParams pAPIParams)
        {

            InfoAPIRequest mAPIRequest = this.PrepareRequest(pAPIParams);

            //InfoAPIResult mAPIResult = (InfoAPIResult)this.GetInfoAPIResult(pAPIParams.app_option);

            //InfoAPIResponse mAPIResponse = this.PrepareResponse(pAPIResult);
            //InfoAPIResponse mAPIResponse = this.PrepareResponse(mAPIResult);
            InfoAPIResponse mAPIResponse = this.PrepareResponse(pAPIParams.app_option);


            Exception err = new Exception();

            WebClient mWebClient = new WebClient();

            this.onAPICallBeforeCall(this, new InfoAPICallBeforeCallEventArgs());

            try
            {

                Uri convertedUri = new Uri(this.APIServerURL);
                var POSTVars = this.GetPostVars();
                var mResponse = await mWebClient.UploadValuesTaskAsync(convertedUri, "POST", POSTVars);

                this.ResponseString = Encoding.Default.GetString(mResponse);

                var client = new RestClient(this.APIServerURL);
                var request = new RestRequest("", Method.POST);
                request.AddParameter("json", this.QueryString);
                request.AddParameter("DebugLevel", this.DebugLevel.ToString());
                request.AddParameter("DebugSection", this.DebugSections);
                //IRestResponse response = client.ExecuteAsync(request,this.);
                //this.ResponseString = response.Content; // raw content as string


                mAPIResponse.Deserialize(this.ResponseString);

                if (!string.IsNullOrEmpty(mAPIResponse.app_option))
                {
                    mAPIResponse.Result.app_option = mAPIResponse.app_option;
                    //pAPIResult.app_option = mAPIResponse.app_option;
                }


                if (mAPIResponse.isResultStateOK())
                {

                    mAPIResponse.Result.Deserialize(mAPIResponse.data);

                    InfoAPICallCompletedEventArgs mAPICallCompletedEventArgs = new InfoAPICallCompletedEventArgs(mAPIResponse);
                    this.onAPICallCompleted(this, mAPICallCompletedEventArgs);
                }
                else
                {

                    InfoAPICallErrorEventArgs mAPICallErrorEventArgs = new InfoAPICallErrorEventArgs();
                    mAPICallErrorEventArgs.error_message = mAPIResponse.error_message;
                    mAPICallErrorEventArgs.error_state = mAPIResponse.error_state;
                    mAPICallErrorEventArgs.error_data = mAPIResponse.Result.error_data;

                    this.onAPICallError(this, mAPICallErrorEventArgs);

                }

            }
            catch (Exception e)
            {
                InfoAPICallErrorEventArgs mAPICallErrorEventArgs = new InfoAPICallErrorEventArgs();
                mAPICallErrorEventArgs.error_state = "API_EXECUTE_ERROR";
                mAPICallErrorEventArgs.error_message = e.Message;

                this.onAPICallError(this, mAPICallErrorEventArgs);

            }
            finally
            {
                this.onAPICallAfterCall(this, new InfoAPICallAfterCallEventArgs());
            }
        }


    }
}
