﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization;

namespace InfoAPILib
{
    /// <summary>
    /// InfoAPIParams klasa obezbeđuje funkcionalnost parametara koji se prosleđuju prilikom poziva API funkciji.
    /// Treba napraviti klasu koja nasleđuje ovu klasu i podesiti je za poziv konkretne API funkcije
    /// </summary>
    [DataContract]
    public class InfoAPIParams : InfoAPIProperties
    {
        private InfoAPIRequest _RequestEnvelope;
        
        public InfoAPIRequest RequestEnvelope {
            get {
                return this._RequestEnvelope;
            }
            set {
                this._RequestEnvelope = value;
                this._RequestEnvelope.app_option = app_option;
            }
        }
        /// <summary>
        /// Format u kome tabelarni podaci koje funkcija vraća treba da budu.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string result_database_format { get; set; }

        [DataMember(EmitDefaultValue = false)]
        public string default_result_database_format { get; protected set; }
        /// <summary>
        /// Dodaj prilog u listu priloga koji se šalju API funkciji.
        /// </summary>
        /// <param name="pAttachment">Prilog koji se dodaje</param>
        //public void AddAttachment (InfoAPIAttachment pAttachment)
        //{
        //    this.Attachments.Add(pAttachment);
        //}

        /// <summary>
        /// Dodaj referencu za tabelu u listu referenci tabela. Pod tabelom se smatra svaki tabelarni podatak, 
        /// odnosno bilo kakva IEnumerable lista podataka. 
        /// 
        /// Struktura sloga tabelarnog podataka može biti proizvoljna. Obično API funkcija definiše nazive 
        /// i strukturu tabela koji joj se mogu poslati.
        /// </summary>
        /// <param name="pTableName">Naziv tabele koja se dodaje kao parametar</param>
        /// <param name="pTableData">Podaci u tabelarnom obliku (bilo kakva IEnumerable lista). Struktura 
        /// 
        /// sloga tabelarnog podataka može biti proizvoljna. Obično API funkcija definiše nazive 
        /// i strukturu tabela koji joj se mogu poslati.</param>
        public void AddTableReference (string pTableName, IEnumerable<object> pTableData)
        {
            this.TableReferences.Add(pTableName, pTableData);
        }


        /// <summary>
        /// Dodaj referencu za tabelu u listu referenci tabela. Pod tabelom se smatra svaki tabelarni podatak, 
        /// odnosno bilo kakva IEnumerable lista podataka. 
        /// 
        /// Struktura sloga tabelarnog podataka može biti proizvoljna. Obično API funkcija definiše nazive 
        /// i strukturu tabela koji joj se mogu poslati.
        /// </summary>
        /// <param name="pTableName">Naziv tabele koja se dodaje kao parametar</param>
        /// <param name="pTableData">Podaci u tabelarnom obliku (bilo kakva IEnumerable lista). Struktura 
        /// 
        /// Type of data is passed as generic type.
        /// 
        /// sloga tabelarnog podataka može biti proizvoljna. Obično API funkcija definiše nazive 
        /// i strukturu tabela koji joj se mogu poslati.</param>
        public void AddTableReference<T>(string pTableName, IEnumerable<object> pTableData)
        {
            this.TableReferences.Add<T>(pTableName, pTableData);
        }

    }
}
