﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization;
using System.Dynamic;
using System.Collections;
using System.Reflection;
using System.IO;

namespace InfoAPILib
{
    public class InfoAPIProperties : InfoAPIBase  {
        [DataMember(EmitDefaultValue = false)]
        public string database_format { get; set; }

        [DataMember(EmitDefaultValue = false)]
        public bool remove_empty_table_fields { get; set; }

        [DataMember(EmitDefaultValue = false, Name = "databaseattachment")]
        public string DataBaseAttachment { get; protected set; }

        public InfoAPIDatabaseAttachment DataBaseAttachmentUnpacked { get; private set; }

        [DataMember(EmitDefaultValue = false)]
        public string paging_sort_exp { get; set; }

        [DataMember(EmitDefaultValue = false)]
        public int paging_selected_page { get; set; }

        [DataMember(EmitDefaultValue = false)]
        public int paging_page_len { get; set; }

        [DataMember(EmitDefaultValue = false, Name = "tablereferences")]
        protected InfoAPITableReferences TableReferences = new InfoAPITableReferences();

        private List<string> ObjDatabaseFormats;

        public bool ShouldSerializeTableReferences()
        {
            return this.TableReferences.items.Count > 0;
        }

        [DataMember(EmitDefaultValue = false)]
        protected InfoAPIAttachments Attachments = new InfoAPIAttachments();


        public InfoAPIProperties()
        {
            this.ObjDatabaseFormats = new List<string> { "PLAIN_OBJ"};
            this.database_format = "PLAIN_OBJ";
            this.DataBaseAttachmentUnpacked = new InfoAPIDatabaseAttachment();
        }


        /// <summary>
        /// Snima database_attachemnt u binarnu datoteku. Zadržava se izvorni format podataka.
        /// Ako su opdaci u PLAON_OBJ formatu onda će biti sačuvani kao JSON tekst.
        /// </summary>
        /// <param name="pFilePathName">Putanja i naziv datoteke u koju treba snimiti podatke</param>
        /// <returns></returns>
        public bool SaveDataBaseAttachmentToFile(string pFilePathName)
        {
            byte[] mData;

            if (this.database_format.ToUpper() != "PLAIN_OBJ")
            {
                mData = Convert.FromBase64String(this.DataBaseAttachment);
            }
            else
            {
                mData = Encoding.ASCII.GetBytes(this.DataBaseAttachment);
            }
            return InfoAPIFile.ByteArrayToFile(pFilePathName, mData);

        }

        public void LoadDatabaseAttachmentFromFile(string pFilePathName)
        {
            byte[] lData = File.ReadAllBytes(pFilePathName);
            //string lDataStr = Encoding.UTF8.GetString(lData);
            string lDataStr = Convert.ToBase64String(lData);
            this.DataBaseAttachment = lDataStr;

        }





        public void PrepareDatabaseAttachment()
        {

            if ( (this.TableReferences.count > 0) && this.ObjDatabaseFormats.Contains(this.database_format) ) {

                this.DataBaseAttachmentUnpacked.LoadFromTableReferences(this.TableReferences);
                this.DataBaseAttachment = this.Formatter.Serialize(DataBaseAttachmentUnpacked);

            }

        }


        public void UnpackDatabaseAttachment()
        {

            if ((this.database_format == "PLAIN_OBJ")) { //&& (this.DataBaseAttachmentUnpacked == null)) { 
                this.DataBaseAttachmentUnpacked = Newtonsoft.Json.JsonConvert.DeserializeObject<InfoAPIDatabaseAttachment>(this.DataBaseAttachment);
            }

        }

        public DatabaseAttachmentTable GetDatabaseAttachmentTable (string pTableName)
        {

            DatabaseAttachmentTable mTable = null;

            this.UnpackDatabaseAttachment();

            if (this.DataBaseAttachmentUnpacked != null) { 
                mTable = this.DataBaseAttachmentUnpacked.items.Where(e => e.table_name == pTableName).SingleOrDefault();
            }
            return mTable;
        }

        // procitaj podatke iz primljenog JSON-a iz tabele pTableName i prepisi u pDestinationTable
        public void ReadTable<T>(string pTableName, IList<T> pDestinationTable)
        {
            // izvadi tabelu iz JSON-a po imenu
            DatabaseAttachmentTable mDatabaseAtachmentTable = this.GetDatabaseAttachmentTable(pTableName);

            // ako tabela postoji u JSON objekat nece biti null
            if (mDatabaseAtachmentTable != null)
            {
                // napuni tu tabelu podaima iz JSON-a.
                mDatabaseAtachmentTable.Populate(pDestinationTable);
            }
        }



        /// <summary>
        /// Serijalizuje objekat
        /// </summary>
        public new string Serialize()
        {
            if (this.DataBaseAttachment == null) { 
                this.PrepareDatabaseAttachment();
            }
            return base.Serialize(false);
        }
        


    }
}
