﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InfoAPILib;
using System.Windows.Forms;
using Microsoft.VisualBasic.FileIO;
using System.IO;

namespace InfoAPIUniversalClient
{

    public delegate void ShowMessageEventHandler (object sender, string pMessage, Boolean pCRLF = true);

    public class InfoAPIUniversalClient
    {
        public string ClientVersion = "0.1.2";
        private InfoAPIHandler APIHandler;
        private InfoAPIParamsUniversal APIParams;
        private InfoAPIResponse APIResponse;
        private string CommandScript;
        public bool SaveLogStatus = false;
        public string Log = "";
        private bool ExitStatus = false;
        private bool ExitOnEndStatus = false;

        public InfoAPIUniversalClient(String pCommandScript,
                                      String pUserName,
                                      String pPassword,
                                      String pClientId,
                                      String pHost)
        {
            this.APIHandler = new InfoAPIHandler();
            this.APIParams = new InfoAPIParamsUniversal();

            this.CommandScript = Path.GetFullPath(pCommandScript);
            this.APIHandler.username = pUserName;
            this.APIHandler.password = pPassword;
            this.APIHandler.client_id = pClientId;
            this.APIHandler.APIServerURL = pHost;

            this.APIHandler.client_version = this.ClientVersion;

            // privremeno podeseno, treba da se ucita iz konfiguracije
            this.APIHandler.client_pk = "87oluavs978bl94vkivfzs967zs3rf";

            //this.APIHandler.client_version = "0.0.1";
            //this.APIHandler.poslovna_godina = tbPoslovnaGodina.Text;
            //this.APIHandler.DebugLevel = 0;
            //this.APIHandler.DebugSections = "SYSTEM,CLASS";
            //this.APIHandler.SerializeIdented = true;
            //this.APIHandler.SerializeIdented = false;

            //this.APIHandler.APIServerURL = tbServerURL.Text;

            this.SaveLogStatus = false;
            this.ExitStatus = false;
            this.ExitOnEndStatus = false;


        }

        public void Run ()
        {
            try
            {
                this.ShowMessage("Loading command file: " + this.CommandScript);
                CommandParser Parser = new CommandParser(this.CommandScript);
                Command Command = null;
                do
                {
                    Command = Parser.GetCommand();
                    if (Command != null)
                    {
                        this.Execute(Command);
                        //Parser.Next();
                    }
                } while (Command != null && this.ExitStatus == false);
            }
            catch (Exception lException)
            {
                //if (this.SaveLogStatus) {
                    this.Log += "\r\n" + lException.Message + "\r\n";
                //}
                throw lException;
            } finally
            {
                this.ShowMessage("\r\n\rAPI call execution is finished.");
                this.SaveLog();

                if (this.ExitStatus || this.ExitOnEndStatus)
                {
                    Application.Exit();
                }

            }
        }

        private void Execute(Command pCommand)
        {
            this.ShowMessage("\r\n > ", false);
            this.ShowMessage(pCommand.CommandLine);
            if (pCommand != null) { 
                string CommandName = pCommand.CommandName.ToUpper();
                switch (CommandName) {

                    case "EXECUTE":
                        this.ExecuteAPICall(pCommand, this);
                        break;

                    case "SET":
                        this.SetOption(pCommand, this);
                        break;

                    case "ADDTABLE":
                        this.AddTable(pCommand, this);
                        break;

                    case "SAVESTATUS":
                        this.SaveStatus(pCommand, this);
                        break;

                    case "LOGGING":
                        this.Logging(pCommand, this);
                        break;

                    case "SHOWDATETIME":
                        this.ShowDateTime(pCommand, this);
                        break;

                    case "HOST":
                        this.SetHost(pCommand, this);
                        break;

                    case "SAVEREQUESTJSON":
                        this.SaveRequestJSON(pCommand, this);
                        break;

                    case "SAVEPARAMSJSON":
                        this.SaveParamsJSON(pCommand, this);
                        break;

                    case "SAVERESPONSEJSON":
                        this.SaveResponseJSON(pCommand, this);
                        break;

                    case "SAVERESULTJSON":
                        this.SaveResultJSON(pCommand, this);
                        break;

                    case "SAVELOG":
                        this.SaveLog();
                        break;

                    case "EXIT":
                        this.ShowMessage("Exiting");
                        this.ExitStatus = true;
                        break;

                    case "EXITONEND":
                        this.ShowMessage("Will exit on end");
                        this.ExitOnEndStatus = true;
                        break;

                    default:
                        throw new Exception("Unknown command: " + pCommand.CommandLine);
                        break;
                }
            }
        }

        private void ShowMessage (string pMessage, bool pCRLF = true)
        {
            this.doShowMessage (this, pMessage, pCRLF);
        }


        public event ShowMessageEventHandler onShowMessage;

        protected virtual void doShowMessage(object sender, string pMessage, bool pCRLF)
        {
            //if (this.SaveLogStatus)
            //{
                this.Log += pMessage;
                if (pCRLF)
                {
                    this.Log += "\r\n";
                }
            //}
            this.onShowMessage?.Invoke(this, pMessage, pCRLF);
        }

        private void SaveRequestJSON(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string lFileName = Path.ChangeExtension(this.CommandScript, ".request.json");
            this.ShowMessage("Saving request JSON to: " + lFileName);
            File.WriteAllText(lFileName, this.APIHandler.QueryString);
        }

        private void SaveParamsJSON(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string lFileName = Path.ChangeExtension(this.CommandScript, ".params.json");
            this.ShowMessage("Saving params JSON to: " + lFileName);
            File.WriteAllText(lFileName, this.APIHandler.ParamsString);
        }

        private void SaveResponseJSON(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string lFileName = Path.ChangeExtension(this.CommandScript, ".response.json");
            this.ShowMessage("Saving response JSON to: " + lFileName);

            File.WriteAllText(lFileName, this.APIHandler.ResponseString);
        }

        private void SaveResultJSON(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string lFileName = Path.ChangeExtension(this.CommandScript, ".result.json");
            this.ShowMessage("Saving result JSON to: " + lFileName);
            File.WriteAllText(lFileName, this.APIResponse.data);
        }

        private void ShowDateTime(Command pCommand, InfoAPIUniversalClient pSender)
        {
            DateTime lDateTime = DateTime.Now;
            this.ShowMessage(lDateTime.ToString());
        }

        private void Logging(Command pCommand, InfoAPIUniversalClient pSender)
        {
            this.SaveLogStatus = true;
            this.ShowMessage("Saving log turned on.");
        }

        private void SaveLog ()
        {
            string lFileName = Path.ChangeExtension(this.CommandScript, ".log");
            this.ShowMessage("\r\nSaving log to: " + lFileName);
            if (this.SaveLogStatus)
            {
                File.WriteAllText(lFileName, this.Log);
            }
        }

        private void SaveStatus (Command pCommand, InfoAPIUniversalClient pSender)
        {
            string lStatus = this.APIResponse.error_state + "\r\n" + this.APIResponse.error_message;
            string lFileName = Path.ChangeExtension(pSender.CommandScript, ".status");
            this.ShowMessage("Saving status to file: " + lFileName + " ... ", false);
            File.WriteAllText(lFileName, lStatus);
            this.ShowMessage(" saved");


        }

        private void SetHost (Command pCommand, InfoAPIUniversalClient pSender)
        {

            string lHost;
            pCommand.Parameters.TryGetValue("1", out lHost);
            if (!String.IsNullOrWhiteSpace(lHost))
            {
                pSender.APIHandler.APIServerURL = lHost;
                this.ShowMessage("HOST set to: " + pSender.APIHandler.APIServerURL);
            } else
            {
                this.ShowMessage("HOST not set");
            }
        }

        private void ExecuteAPICall(Command pCommand, InfoAPIUniversalClient pSender)
        {
            Application.DoEvents();

            this.ShowMessage("Contacting API service at " + pSender.APIHandler.APIServerURL + " ... ", false);
            this.APIResponse = pSender.APIHandler.CallAPI(pSender.APIParams);
            this.ShowMessage("received API response");
            this.ShowMessage("Status: " + pSender.APIResponse.error_state + " - " + pSender.APIResponse.error_message);

        }

        private void SetOption(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string pParam1 = pCommand.Parameters["1"].ToUpper();
            if (pParam1 != "DATA")
            {
                pSender.SetEnvelopeParameter(pCommand, pSender);
            } else
            {
                pSender.SetParameters(pCommand, pSender);
            }
        }

        private void SetEnvelopeParameter(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string mParameter = pCommand.Parameters["1"].ToUpper();
            string mValue = pCommand.Parameters["2"];
            switch (mParameter) {
                case "APP_OPTION":
                    pSender.APIParams.app_option = mValue;
                    pSender.ShowMessage("APP_OPTION set to: " + pSender.APIParams.app_option);
                    break;
                default:
                    throw new Exception ("Unknown SET parameter: " + pCommand.CommandLine);
                    break;
            }

        }

        private void SetParameters(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string mParameter = pCommand.Parameters["2"].ToUpper();
            string mValue = pCommand.Parameters["3"];
            this.ShowMessage("Setting data." + mParameter + " to value " + mValue + " not supported yet.");
        }


        private void AddTable(Command pCommand, InfoAPIUniversalClient pSender)
        {
            string mType;
            pCommand.Parameters.TryGetValue("type", out mType);
            
            if (String.IsNullOrWhiteSpace(mType)) {
              throw new Exception("Parameter -type is required!");
            }

            mType = mType.Trim().ToUpper();

            switch (mType)
            {
                case "CSV":
                    this.LoadCSVTable(pCommand, pSender);
                    break;
                default:
                    throw new Exception("Uknown table type: " + mType);
            }
            
        }

        private void LoadCSVTable(Command pCommand, InfoAPIUniversalClient pSender)
        {

            string mFile;
            pCommand.Parameters.TryGetValue("file", out mFile);

            if (String.IsNullOrWhiteSpace(mFile))
            {
                throw new Exception("Parameter -file is required!");
            }

            mFile = Path.GetFullPath(mFile);

            if (!File.Exists(mFile))
            {
                throw new Exception("File " + mFile + " does not exist!");
            }


            string mTableName; // = pCommand.Parameters.TryGetValue ("name", out "").ToUpper();
            pCommand.Parameters.TryGetValue("name", out mTableName);
            if (String.IsNullOrWhiteSpace(mTableName))
            {
                mTableName = Path.GetFileNameWithoutExtension (mFile);
            }

            this.ShowMessage("Loading table " + mTableName + " from " + mFile + " ... ", false);

            InfoAPIDynamicTable mTable = new InfoAPILib.InfoAPIDynamicTable();


            using (TextFieldParser mCSVParser = new TextFieldParser(mFile))
            {
                mCSVParser.TextFieldType = FieldType.Delimited;
                mCSVParser.SetDelimiters(",");

                Boolean lIsFirstRow = true;

                while (!mCSVParser.EndOfData)
                {
                    string[] lFields = mCSVParser.ReadFields();
                    if (lIsFirstRow) {
                        mTable.SetFields (lFields);
                        lIsFirstRow = false;
                    } else {
                        mTable.AddRow (lFields);
                    }
                }
            }

            pSender.APIParams.AddTableReference("tabela", mTable);

            this.ShowMessage("loaded " + mTable.Count() + " records.");

            //this.ShowMessage("Loading table " + mFile + " of type " + mType + " not supported yet.");
        }

    }
}

